@@ -2,7 +2,7 @@ use core::panic;
 use std::{
     collections::{HashMap, HashSet},
     hash::Hash,
-    sync::{Arc, RwLock},
+    sync::{Arc, Mutex, MutexGuard},
     usize,
 };
 
@@ -17,9 +17,9 @@ basic information.
  */
 pub struct Index<T> {
     /// Additional information about documents.
-    pub docs: HashMap<T, Arc<RwLock<DocumentDetails<T>>>>,
+    pub docs: HashMap<T, Arc<Mutex<DocumentDetails<T>>>>,
     /// Inverted index root node.
-    pub root: Arc<RwLock<InvertedIndexNode<T>>>,
+    pub root: Arc<Mutex<InvertedIndexNode<T>>>,
     /// Additional information about indexed fields in all documents.
     pub fields: Vec<FieldDetails>,
 }
@@ -34,7 +34,9 @@ pub fn create_index<T>(fields_num: usize) -> Index<T> {
     let fields: Vec<FieldDetails> = vec![FieldDetails { sum: 0, avg: 0_f64 }; fields_num];
     Index {
         docs: HashMap::new(),
-        root: Arc::new(RwLock::new(create_inverted_index_node(&0))),
+        root: Arc::new(Mutex::new(create_inverted_index_node(
+            &char::from_u32(0).unwrap(),
+        ))),
         fields,
     }
 }
@@ -42,7 +44,7 @@ pub fn create_index<T>(fields_num: usize) -> Index<T> {
 Document Details object stores additional information about documents.
  * typeparam `T` Document key.
  */
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Debug, PartialEq)]
 pub struct DocumentDetails<T> {
     /**
     Document key. It can be a simple unique ID or a direct reference to original document.
@@ -58,16 +60,16 @@ pub struct DocumentDetails<T> {
 Document pointer contains information about term frequency for a document.
 * typeparam `T` Document key.
 */
-#[derive(Clone, Debug)]
+#[derive(Debug)]
 pub struct DocumentPointer<T> {
     /**
     Next DocumentPointer in the intrusive linked list.
      */
-    pub next: Option<Arc<RwLock<DocumentPointer<T>>>>,
+    pub next: Option<Arc<Mutex<DocumentPointer<T>>>>,
     /**
     Reference to a DocumentDetailsobject that is used for this document.
      */
-    pub details: Arc<RwLock<DocumentDetails<T>>>,
+    pub details: Arc<Mutex<DocumentDetails<T>>>,
     /**
     Term frequency in each field.
      */
@@ -79,29 +81,29 @@ Inverted Index Node.
 Inverted index is implemented with a [trie](https://en.wikipedia.org/wiki/Trie) data structure.
  * typeparam `T` Document key.
 */
-#[derive(Clone, Debug)]
+#[derive(Debug)]
 pub struct InvertedIndexNode<T> {
     /**
     Char code is used to store keys in the trie data structure.
      */
-    pub char_code: u32,
+    pub char: char,
     /**
     Next InvertedIndexNode in the intrusive linked list.
      */
-    pub next: Option<Arc<RwLock<InvertedIndexNode<T>>>>,
+    pub next: Option<Arc<Mutex<InvertedIndexNode<T>>>>,
     /**
     Linked list of children {@link InvertedIndexNode}.
      */
-    pub first_child: Option<Arc<RwLock<InvertedIndexNode<T>>>>,
+    pub first_child: Option<Arc<Mutex<InvertedIndexNode<T>>>>,
     /**
     Linked list of documents associated with this node.
      */
-    pub first_doc: Option<Arc<RwLock<DocumentPointer<T>>>>,
+    pub first_doc: Option<Arc<Mutex<DocumentPointer<T>>>>,
 }
 
 impl<T> PartialEq for InvertedIndexNode<T> {
     fn eq(&self, other: &InvertedIndexNode<T>) -> bool {
-        other.char_code == self.char_code
+        other.char == self.char
     }
 }
 
@@ -126,9 +128,9 @@ Creates inverted index node.
  * `charCode` Char code.
  returns InvertedIndexNode instance.
  */
-fn create_inverted_index_node<T>(char_code: &u32) -> InvertedIndexNode<T> {
+fn create_inverted_index_node<T>(char: &char) -> InvertedIndexNode<T> {
     InvertedIndexNode {
-        char_code: char_code.to_owned(),
+        char: char.to_owned(),
         next: None,
         first_child: None,
         first_doc: None,
@@ -143,18 +145,17 @@ Finds inverted index node that matches the `term`.
 returns Inverted index node that contains `term` or an `undefined` value.
  */
 pub fn find_inverted_index_node<T>(
-    node: Arc<RwLock<InvertedIndexNode<T>>>,
+    node: Arc<Mutex<InvertedIndexNode<T>>>,
     term: &str,
-) -> Option<Arc<RwLock<InvertedIndexNode<T>>>> {
-    let chars = term.chars();
+) -> Option<Arc<Mutex<InvertedIndexNode<T>>>> {
     let mut node_iteration = Some(node);
-    for char in chars {
+    for char in term.chars() {
         if node_iteration.is_none() {
             break;
         }
-        node_iteration = find_inverted_index_node_child_nodes_by_char_code(
-            &node_iteration.unwrap(),
-            char as u32,
+        node_iteration = find_inverted_index_node_child_nodes_by_char(
+            &node_iteration.unwrap().lock().unwrap(),
+            &char,
         );
     }
     node_iteration
@@ -167,21 +168,25 @@ Finds inverted index child node with matching `char`.
  * `charCode` Char code.
 returns Matching InvertedIndexNode or `undefined`.
  */
-fn find_inverted_index_node_child_nodes_by_char_code<T>(
-    node: &Arc<RwLock<InvertedIndexNode<T>>>,
-    char_code: u32,
-) -> Option<Arc<RwLock<InvertedIndexNode<T>>>> {
-    let node_unwrapped = node.read().unwrap();
-    let child = &node_unwrapped.first_child;
+fn find_inverted_index_node_child_nodes_by_char<T>(
+    node_lock: &MutexGuard<InvertedIndexNode<T>>,
+    char: &char,
+) -> Option<Arc<Mutex<InvertedIndexNode<T>>>> {
+    let child = &node_lock.first_child;
     if child.is_none() {
         return None;
     }
-    let mut iter = Arc::clone(node_unwrapped.first_child.as_ref().unwrap());
+    let mut iter = Arc::clone(node_lock.first_child.as_ref().unwrap());
     loop {
-        if iter.read().unwrap().char_code == char_code {
-            return Some(Arc::clone(&iter));
+        let new_iter;
+        if let Ok(l) = &iter.lock() {
+            if &l.char == char {
+                return Some(Arc::clone(&iter));
+            }
+            new_iter = l.next.as_ref().map(|c| Arc::clone(&c));
+        } else {
+            panic!()
         }
-        let new_iter = iter.read().unwrap().next.as_ref().map(|c| Arc::clone(&c));
         match new_iter {
             Some(n) => {
                 iter = n;
@@ -198,15 +203,15 @@ Adds inverted index child node.
  * `child` Child node to add.
  */
 fn add_inverted_index_child_node<T: Clone>(
-    parent: &Arc<RwLock<InvertedIndexNode<T>>>,
-    child: Arc<RwLock<InvertedIndexNode<T>>>,
+    parent: &mut MutexGuard<InvertedIndexNode<T>>,
+    child: Arc<Mutex<InvertedIndexNode<T>>>,
 ) {
-    //-> Arc<RwLock<InvertedIndexNode<T>>>
-    if let Some(first) = parent.read().unwrap().first_child.clone() {
-        child.write().unwrap().next = Some(first);
+    //-> Arc<Mutex<InvertedIndexNode<T>>>
+    if let Some(first) = parent.first_child.clone() {
+        child.lock().unwrap().next = Some(first);
     }
     //let c=  parent.clone();
-    parent.write().unwrap().first_child = Some(child);
+    parent.first_child = Some(child);
     //Arc::clone(&parent.borrow().first_child.unwrap())
 }
 
@@ -223,13 +228,14 @@ typeparam `D` Document type.
  * `doc` Document.
  */
 fn add_inverted_index_doc<T: Clone>(
-    node: &Arc<RwLock<InvertedIndexNode<T>>>,
+    node: Arc<Mutex<InvertedIndexNode<T>>>,
     mut doc: DocumentPointer<T>,
 ) {
-    if let Some(first) = &node.write().unwrap().first_doc {
+    let mut node_locked = node.lock().unwrap();
+    if let Some(first) = &node_locked.first_doc {
         doc.next = Some(Arc::clone(first));
     }
-    node.write().unwrap().first_doc = Some(Arc::new(RwLock::new(doc)));
+    node_locked.first_doc = Some(Arc::new(Mutex::new(doc)));
 }
 
 /**
@@ -253,7 +259,7 @@ pub fn add_document_to_index<T: Eq + Hash + Copy, D>(
     let mut term_counts: HashMap<String, Vec<usize>> = HashMap::new();
     let mut all_terms: Vec<String> = Vec::new();
     for i in 0..fields.len() {
-        match field_accessors.get(i).unwrap()(&doc) {
+        match field_accessors[i](&doc) {
             None => {
                 field_length.push(0);
             }
@@ -268,8 +274,8 @@ pub fn add_document_to_index<T: Eq + Hash + Copy, D>(
                 let mut filtered_terms_count = 0;
                 for mut term in terms {
                     term = filter(&term);
-                    all_terms.push(term.to_owned());
-                    if term.as_str() != "" {
+                    if !term.is_empty() {
+                        all_terms.push(term.to_owned());
                         filtered_terms_count += 1;
                         let counts = term_counts.get_mut(&term);
                         match counts {
@@ -278,8 +284,8 @@ pub fn add_document_to_index<T: Eq + Hash + Copy, D>(
                                 new_count[i] += 1;
                                 term_counts.insert(term, new_count);
                             }
-                            _ => {
-                                counts.unwrap()[i] += 1;
+                            Some(c) => {
+                                c[i] += 1;
                             }
                         }
                     }
@@ -292,18 +298,20 @@ pub fn add_document_to_index<T: Eq + Hash + Copy, D>(
         }
     }
 
-    let details = Arc::new(RwLock::new(DocumentDetails { key, field_length }));
-
-    docs.insert(key, Arc::clone(&details));
+    docs.insert(
+        key,
+        Arc::new(Mutex::new(DocumentDetails { key, field_length })),
+    );
     for term in all_terms {
         let mut node = Arc::clone(&index.root);
+
         for (i, char) in term.chars().enumerate() {
-            if node.read().unwrap().first_child.is_none() {
+            if node.lock().unwrap().first_child.is_none() {
                 node = create_inverted_index_nodes(node, &term, &i);
                 break;
             }
             let next_node =
-                find_inverted_index_node_child_nodes_by_char_code(&Arc::clone(&node), char as u32);
+                find_inverted_index_node_child_nodes_by_char(&node.lock().unwrap(), &char);
             match next_node {
                 None => {
                     node = create_inverted_index_nodes(node, &term, &i);
@@ -315,11 +323,11 @@ pub fn add_document_to_index<T: Eq + Hash + Copy, D>(
             }
         }
         add_inverted_index_doc(
-            &node,
+            node,
             DocumentPointer {
                 next: None,
-                details: Arc::clone(&details),
-                term_frequency: term_counts.get(&term).unwrap().to_owned(),
+                details: Arc::clone(&Arc::clone(docs.get(&key).unwrap())),
+                term_frequency: term_counts[&term].to_owned(),
             },
         )
     }
@@ -335,22 +343,27 @@ Creates inverted index nodes for the `term` starting from the `start` character.
 
  */
 fn create_inverted_index_nodes<T: Clone>(
-    mut parent: Arc<RwLock<InvertedIndexNode<T>>>,
+    mut parent: Arc<Mutex<InvertedIndexNode<T>>>,
     term: &str,
     start: &usize,
-) -> Arc<RwLock<InvertedIndexNode<T>>> {
+) -> Arc<Mutex<InvertedIndexNode<T>>> {
     for (i, char) in term.chars().enumerate() {
         if &i < start {
             continue;
         }
-        let new_node: InvertedIndexNode<T> = create_inverted_index_node(&(char as u32));
-        add_inverted_index_child_node(&parent, Arc::new(RwLock::new(new_node)));
-        let new_parent = match &parent.read().unwrap().first_child {
-            None => {
-                panic!();
-            }
-            Some(x) => Arc::clone(x),
-        };
+        let new_node: InvertedIndexNode<T> = create_inverted_index_node(&char);
+        let new_parent;
+        if let Ok(mut l) = parent.lock() {
+            add_inverted_index_child_node(&mut l, Arc::new(Mutex::new(new_node)));
+            new_parent = match &l.first_child {
+                None => {
+                    panic!();
+                }
+                Some(x) => Arc::clone(x),
+            };
+        } else {
+            panic!()
+        }
         parent = new_parent;
     }
     parent
@@ -375,11 +388,11 @@ pub fn remove_document_from_index<T: Hash + Eq + Copy>(
     let mut remove_key = false;
     if let Some(doc_details) = doc_details_option {
         removed.insert((&key).to_owned());
-        let details = &doc_details.read().unwrap();
+        let details = &doc_details.lock().unwrap();
         remove_key = true;
         let new_len = (index.docs.len() - 1) as f64;
         for i in 0..fields.len() {
-            let field_length = details.field_length.get(i).unwrap();
+            let field_length = &details.field_length[i];
             if field_length > &0 {
                 let field = fields.get_mut(i);
                 if let Some(f) = field {
@@ -410,20 +423,20 @@ Recursively cleans up removed documents from the index.
  * `removed` Set of removed document ids.
 */
 fn vacuum_node<T: Hash + Eq>(
-    node: Arc<RwLock<InvertedIndexNode<T>>>,
+    node: Arc<Mutex<InvertedIndexNode<T>>>,
     removed: &mut HashSet<T>,
 ) -> usize {
-    let mut prev_pointer: Option<Arc<RwLock<DocumentPointer<T>>>> = None;
-    let mut pointer_option = (&node.write().unwrap().first_doc).clone();
+    let mut prev_pointer: Option<Arc<Mutex<DocumentPointer<T>>>> = None;
+    let mut pointer_option = (&node.lock().unwrap().first_doc).clone();
     while let Some(pointer) = pointer_option {
-        let pb = &pointer.read().unwrap();
-        if removed.contains(&pb.details.read().unwrap().key) {
+        let pb = &pointer.lock().unwrap();
+        if removed.contains(&pb.details.lock().unwrap().key) {
             match &prev_pointer {
                 None => {
-                    node.write().unwrap().first_doc = pb.next.clone();
+                    node.lock().unwrap().first_doc = pb.next.clone();
                 }
                 Some(prev) => {
-                    prev.write().unwrap().next = pb.next.clone();
+                    prev.lock().unwrap().next = pb.next.clone();
                 }
             }
         } else {
@@ -432,25 +445,25 @@ fn vacuum_node<T: Hash + Eq>(
         pointer_option = (&pb.next).clone();
     }
 
-    let mut prev_child: Option<Arc<RwLock<InvertedIndexNode<T>>>> = None;
+    let mut prev_child: Option<Arc<Mutex<InvertedIndexNode<T>>>> = None;
     let mut ret = 0;
-    if node.read().unwrap().first_doc.is_some() {
+    if node.lock().unwrap().first_doc.is_some() {
         ret = 1;
     }
 
-    let mut child_option = (&node.read().unwrap().first_child).clone();
+    let mut child_option = (&node.lock().unwrap().first_child).clone();
     while let Some(child) = child_option {
         let r = vacuum_node(Arc::clone(&child), removed);
         ret |= r;
-        let child_unwrapped = child.read().unwrap();
+        let child_unwrapped = child.lock().unwrap();
         if r == 0 {
             // subtree doesn't have any documents, remove this node
             match &prev_child {
                 Some(prev) => {
-                    prev.write().unwrap().next = child_unwrapped.next.clone();
+                    prev.lock().unwrap().next = child_unwrapped.next.clone();
                 }
                 None => {
-                    node.write().unwrap().first_child = child_unwrapped.next.clone();
+                    node.lock().unwrap().first_child = child_unwrapped.next.clone();
                 }
             }
         } else {
@@ -461,6 +474,25 @@ fn vacuum_node<T: Hash + Eq>(
     ret
 }
 
+/**
+    Count the amount of nodes of the index.
+    returns the amount, including root node. Which means count will alway be greater than 0
+*/
+pub fn count_nodes<T>(idx: &Index<T>) -> i32 {
+    fn count_nodes_recursively<T>(node: &Arc<Mutex<InvertedIndexNode<T>>>) -> i32 {
+        let mut count = 1;
+        let n = node.lock().unwrap();
+        if let Some(first) = &n.first_child {
+            count += count_nodes_recursively(first);
+        }
+        if let Some(next) = &n.next {
+            count += count_nodes_recursively(next);
+        }
+        count
+    }
+    count_nodes_recursively(&idx.root)
+}
+
 #[cfg(test)]
 mod tests {
 
@@ -506,35 +538,29 @@ mod tests {
                 field_length: vec![3],
                 key: 1 as usize,
             };
-            assert_eq!(&*added_doc.read().unwrap(), &e);
+            assert_eq!(&*added_doc.lock().unwrap(), &e);
             assert_eq!(index.fields[0], FieldDetails { avg: 3_f64, sum: 3 });
-            let root = &index.root.read().unwrap();
-            assert_eq!(&root.char_code, &0);
+            let root = &index.root.lock().unwrap();
+            assert_eq!(&root.char, &char::from_u32(0).unwrap());
             assert_eq!(&root.next.is_none(), &true);
             assert_eq!(&root.first_doc.is_none(), &true);
 
-            let first_child = &root.first_child.as_ref().unwrap().read().unwrap();
-            assert_eq!(&first_child.char_code, &(99 as u32));
+            let first_child = &root.first_child.as_ref().unwrap().lock().unwrap();
+            assert_eq!(&first_child.char, &(char::from_u32(99).unwrap()));
             assert_eq!(&first_child.first_child.is_none(), &true);
 
-            let first_child_next = &first_child.next.as_ref().unwrap().read().unwrap();
-            assert_eq!(&first_child_next.char_code, &(98 as u32));
+            let first_child_next = &first_child.next.as_ref().unwrap().lock().unwrap();
+            assert_eq!(&first_child_next.char, &(char::from_u32(98).unwrap()));
             assert_eq!(&first_child.first_child.is_none(), &true);
-            let first_child_first_doc = &first_child.first_doc.as_ref().unwrap().read().unwrap();
+            let first_child_first_doc = &first_child.first_doc.as_ref().unwrap().lock().unwrap();
             assert_eq!(&first_child_first_doc.term_frequency, &vec![1 as usize]);
-            assert_eq!(&*first_child_first_doc.details.read().unwrap(), &e);
+            assert_eq!(&*first_child_first_doc.details.lock().unwrap(), &e);
             assert_eq!(&first_child_first_doc.next.is_none(), &true);
             assert_eq!(&first_child_first_doc.next.is_none(), &true);
 
             assert_eq!(
-                &first_child_next
-                    .next
-                    .as_ref()
-                    .unwrap()
-                    .read()
-                    .unwrap()
-                    .char_code,
-                &(97 as u32)
+                &first_child_next.next.as_ref().unwrap().lock().unwrap().char,
+                &(char::from_u32(97).unwrap())
             );
 
             // TODO test more properties
@@ -573,39 +599,59 @@ mod tests {
             );
             assert_eq!(index.docs.len(), 2);
             assert_eq!(
-                &*index.docs.get(&doc_1.id).as_ref().unwrap().read().unwrap(),
+                &*index.docs.get(&doc_1.id).as_ref().unwrap().lock().unwrap(),
                 &DocumentDetails {
                     field_length: vec![3],
                     key: 1 as usize
                 }
             );
             assert_eq!(
-                &*index.docs.get(&doc_2.id).as_ref().unwrap().read().unwrap(),
+                &*index.docs.get(&doc_2.id).as_ref().unwrap().lock().unwrap(),
                 &DocumentDetails {
                     field_length: vec![3],
                     key: 2 as usize
                 }
             );
             assert_eq!(index.fields[0], FieldDetails { avg: 3_f64, sum: 6 });
-            let root = &index.root.read().unwrap();
-            assert_eq!(&root.char_code, &0);
+            let root = &index.root.lock().unwrap();
+            assert_eq!(&root.char, &char::from_u32(0).unwrap());
             assert_eq!(&root.next.is_none(), &true);
             assert_eq!(&root.first_doc.is_none(), &true);
 
-            let first_child = &root.first_child.as_ref().unwrap().read().unwrap();
-            assert_eq!(&first_child.char_code, &(100 as u32));
+            let first_child = &root.first_child.as_ref().unwrap().lock().unwrap();
+            assert_eq!(&first_child.char, &char::from_u32(100).unwrap());
             assert_eq!(&first_child.first_child.is_none(), &true);
 
-            let first_child_next = &first_child.next.as_ref().unwrap().read().unwrap();
-            assert_eq!(&first_child_next.char_code, &(99 as u32));
+            let first_child_next = &first_child.next.as_ref().unwrap().lock().unwrap();
+            assert_eq!(&first_child_next.char, &char::from_u32(99).unwrap());
             assert_eq!(&first_child.first_child.is_none(), &true);
-            let nested_next = first_child_next.next.as_ref().unwrap().read().unwrap();
-            assert_eq!(&nested_next.char_code, &(98 as u32));
-            let nested_nested_next = &nested_next.next.as_ref().unwrap().read().unwrap();
-            assert_eq!(&nested_nested_next.char_code, &(97 as u32));
+            let nested_next = first_child_next.next.as_ref().unwrap().lock().unwrap();
+            assert_eq!(&nested_next.char, &char::from_u32(98).unwrap());
+            let nested_nested_next = &nested_next.next.as_ref().unwrap().lock().unwrap();
+            assert_eq!(&nested_nested_next.char, &char::from_u32(97).unwrap());
 
             // dont test all the properties
         }
+
+        #[test]
+        fn it_should_ignore_empty_tokens() {
+            let field_accessors: Vec<FieldAccessor<Doc>> =
+                vec![field_accessor as fn(doc: &Doc) -> Option<&str>];
+
+            let mut index = create_index::<usize>(1);
+            let doc_1 = Doc {
+                id: 1,
+                text: "a  b".to_string(), // double space could introduce empty tokens
+            };
+            add_document_to_index(
+                &mut index,
+                &field_accessors,
+                tokenizer,
+                filter,
+                doc_1.id,
+                doc_1.clone(),
+            );
+        }
     }
 
     mod delete {
@@ -631,9 +677,9 @@ mod tests {
             assert_eq!(idx.fields.get(0).unwrap().avg.is_nan(), true);
 
             assert_eq!(
-                *idx.root.read().unwrap(),
+                *idx.root.lock().unwrap(),
                 InvertedIndexNode {
-                    char_code: 0,
+                    char: char::from_u32(0).unwrap(),
                     first_child: None,
                     first_doc: None,
                     next: None
@@ -642,11 +688,12 @@ mod tests {
         }
     }
     mod find {
+
         use super::*;
 
-        fn create(char: u32) -> Arc<RwLock<InvertedIndexNode<String>>> {
+        fn create(char: char) -> Arc<Mutex<InvertedIndexNode<String>>> {
             let node: InvertedIndexNode<String> = create_inverted_index_node(&char);
-            Arc::new(RwLock::new(node))
+            Arc::new(Mutex::new(node))
         }
 
         mod char_code {
@@ -654,31 +701,35 @@ mod tests {
 
             #[test]
             fn it_should_find_undefined_children_if_none() {
-                let node = create(0);
-                let c = find_inverted_index_node_child_nodes_by_char_code(&node, 0);
+                let node = create('x');
+                let c = find_inverted_index_node_child_nodes_by_char(&node.lock().unwrap(), &'x');
                 assert_eq!(c.is_none(), true);
             }
 
             #[test]
             fn it_should_find_existing() {
-                let p = create(0);
-                let c1 = create(1);
-                let c2 = create(2);
-                add_inverted_index_child_node(&p, Arc::clone(&c1));
-                add_inverted_index_child_node(&p, Arc::clone(&c2));
+                let p = create('x');
+                let c1 = create('y');
+                let c2 = create('z');
+                add_inverted_index_child_node(&mut p.lock().unwrap(), Arc::clone(&c1));
+                add_inverted_index_child_node(&mut p.lock().unwrap(), Arc::clone(&c2));
                 assert_eq!(
-                    *find_inverted_index_node_child_nodes_by_char_code(&p, 1)
-                        .unwrap()
-                        .read()
-                        .unwrap(),
-                    *c1.read().unwrap()
+                    std::ptr::eq(
+                        find_inverted_index_node_child_nodes_by_char(&p.lock().unwrap(), &'y')
+                            .unwrap()
+                            .as_ref(),
+                        c1.as_ref()
+                    ),
+                    true
                 );
                 assert_eq!(
-                    *find_inverted_index_node_child_nodes_by_char_code(&p, 2)
-                        .unwrap()
-                        .read()
-                        .unwrap(),
-                    *c2.read().unwrap()
+                    std::ptr::eq(
+                        find_inverted_index_node_child_nodes_by_char(&p.lock().unwrap(), &'z')
+                            .unwrap()
+                            .as_ref(),
+                        c2.as_ref()
+                    ),
+                    true
                 );
             }
         }
@@ -687,21 +738,85 @@ mod tests {
             use super::*;
             #[test]
             fn it_should_find() {
-                let p = create(0);
-                let a = create(97);
-                let b = create(98);
-                let c = create(99);
-                add_inverted_index_child_node(&p, Arc::clone(&a));
-                add_inverted_index_child_node(&a, Arc::clone(&b));
-                add_inverted_index_child_node(&b, Arc::clone(&c));
+                let p = create('x');
+                let a = create('a');
+                let b = create('b');
+                let c = create('c');
+                add_inverted_index_child_node(&mut p.lock().unwrap(), Arc::clone(&a));
+                add_inverted_index_child_node(&mut a.lock().unwrap(), Arc::clone(&b));
+                add_inverted_index_child_node(&mut b.lock().unwrap(), Arc::clone(&c));
                 assert_eq!(
-                    *find_inverted_index_node(p, &"abc".to_string())
-                        .unwrap()
-                        .read()
-                        .unwrap(),
-                    *c.read().unwrap()
+                    std::ptr::eq(
+                        find_inverted_index_node(p, &"abc").unwrap().as_ref(),
+                        c.as_ref()
+                    ),
+                    true
                 );
             }
         }
+
+        mod count {
+            use super::*;
+
+            #[test]
+            fn it_should_count_nodes() {
+                let field_accessors: Vec<FieldAccessor<Doc>> =
+                    vec![field_accessor as fn(doc: &Doc) -> Option<&str>];
+
+                let mut index = create_index::<usize>(1);
+                let doc = Doc {
+                    id: 1,
+                    text: "abc".to_string(),
+                };
+                let doc_2 = Doc {
+                    id: 1,
+                    text: "abe".to_string(),
+                };
+                add_document_to_index(&mut index, &field_accessors, tokenizer, filter, doc.id, doc);
+                add_document_to_index(
+                    &mut index,
+                    &field_accessors,
+                    tokenizer,
+                    filter,
+                    doc_2.id,
+                    doc_2,
+                );
+
+                assert_eq!(count_nodes(&index), 5); //
+            }
+
+            #[test]
+            fn it_should_count_nodes_2() {
+                let field_accessors: Vec<FieldAccessor<Doc>> =
+                    vec![field_accessor as fn(doc: &Doc) -> Option<&str>];
+
+                let mut index = create_index::<usize>(1);
+                let doc = Doc {
+                    id: 1,
+                    text: "ab cd".to_string(),
+                };
+                let doc_2 = Doc {
+                    id: 1,
+                    text: "ab ef".to_string(),
+                };
+                add_document_to_index(&mut index, &field_accessors, tokenizer, filter, doc.id, doc);
+                add_document_to_index(
+                    &mut index,
+                    &field_accessors,
+                    tokenizer,
+                    filter,
+                    doc_2.id,
+                    doc_2,
+                );
+
+                assert_eq!(count_nodes(&index), 7); //
+            }
+
+            #[test]
+            fn it_should_count_nodes_empty() {
+                let index = create_index::<usize>(1);
+                assert_eq!(count_nodes(&index), 1); // 1 for root
+            }
+        }
     }
 }
